///////////////////////////////////////////////////////////////////////////////
//	
//								Nano OS	Version	1.0
//						Copyright 2009 xxxxxxx, xxxxxxx
//	File:	autoinit.c
//	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
//	Description: Perform initialization of C variables
//	
///////////////////////////////////////////////////////////////////////////////

#define ALIGN_MASK 0x7
#define ALIGN_PTR(ptr) \
   ((unsigned *)(((unsigned)ptr + ALIGN_MASK) & ~ALIGN_MASK))

typedef void (*PTRFUNC)();
extern far const PTRFUNC __pinit__[];

#ifndef _USE_STD_LIBS

	asm("OS_SYSTEM_STACK_SIZE	.set	1024");
	asm("args: 	.sect  \".args\""); 	// address of arguments space
	asm("	.global ___bss__");			// Beginning of BSS section
	asm("	.global ___end__");			// Beginning of BSS section
	asm("	.global _main");			// Beginning of OS C functions
	asm("	.global _SYSTEM_STACK");
	asm"_SYSTEM_STACK_TOP	.usect ".stack", OS_SYSTEM_STACK_SIZE, 4");

#pragma CODE_SECTION(c_int00,".sysinit");
//#pragma DATA_SECTION (heap_data, ".heap");
//extern UINT8 heap_data[];

extern void interrupt c_int00()
{
	asm("	b 		atomic");
	asm("	zero 	B0");
	asm("	mvc 	B0,IER");
	asm("	nop 	3");

	asm("atomic:	");

		// Next setup the stack
	asm("	mvkl 	_SYSTEM_STACK-4, SP");
	asm("	mvkh 	_SYSTEM_STACK-4, SP");

		// SET UP THE GLOBAL PAGE POINTER IN B14.
	asm("	mvkl ___bss__,DP");
	asm("	mvkh ___bss__,DP");

		// SET UP FLOATING POINT REGISTERS FOR C67xx
	asm("	mvk 0,B3");
	asm("	mvc B3,FADCR");
	asm("	mvc B3,FMCR");

		// INITIALIZE CONTROL REGISTERS
	asm("	mvk 0,B3");		// addressing mode register
	asm("	mvc B3,AMR");	// interrupt enable register
	asm("	mvc B3,CSR");

		// GET THE POINTER TO THE AUTOINITIALIZATION TABLES INTO THE FIRST 
		// ARGUMENT REGISTER (A4), AND CALL A FUNCTION TO PERFORM
		// AUTOINITIALIZATION.
   	asm("	.global cinit");
   	asm("	mvkl 	cinit,A4");
   	asm("	mvkh 	cinit,A4");

		// PASS THE CURRENT DP TO THE AUTOINITIALIZATION ROUTINE
	asm("	mv 	DP,B4");
		
		// CALL THE AUTOINITIALIZATION ROUTINE.                                   
   	asm("	.global __auto_init");
	asm("	mvkl __auto_init, B0");
   	asm("	mvkh __auto_init, B0");
   	asm("	b    B0");
   	asm("	mvkl boot_ret, B3");
   	asm("	mvkh boot_ret, B3");
   	asm("	nop  3");

	asm("boot_ret:	");
	asm("	mvkl args,A0");
	asm("	mvkh args,A0");
	asm("	ldw *+A0[2],A6");
	asm("	ldw *+A0[1],B4");
	asm("	ldw *+A0[0],A4");

		// Call the C function 'os_cinit'
	asm("	add  	-12,SP,SP");			// C calling convention
	asm("	mvkl 	_main,B0");      // load destination function address to B0
	asm("	mvkh 	_main,B0");
	asm("	B 		B0");				// start branch to destination function
	asm("	nop	5");
}

void _auto_init(const void *cinit, const unsigned int dp)
{
   const unsigned int *recptr = cinit;
   int length;

   if ((int)recptr != -1)
   {
      while ((length = *recptr++) != 0)
      {
         if (length < 0)
         {
            int i;

            for (i = -length; i != 0; i-=4) 
            {
                unsigned int *to  = (unsigned int *) *recptr++;
                *to += dp; 
            }
            recptr = ALIGN_PTR(recptr);
         }
         else
         {
            char *to   = (void*) *recptr++; 
            char *from = (void*) recptr; 
			int templ = length;

			while(templ--) *to++ = *from++;

            recptr  = ALIGN_PTR(from);
         }
      }
   }

   /*------------------------------------------------------------------------*/
   /* Process Pinit table.                                                   */
   /* consists of pointers to init functions.                                */
   /* section is NULL terminated                                             */
   /* pointer is = -1 if section does not exist.                             */
   /*------------------------------------------------------------------------*/
   if ((int)__pinit__ != -1)
   {
      int i = 0;
      while (__pinit__[i] != 0 )
         __pinit__[i++]();
   }
}

#endif // _USE_STD_LIBS
