///////////////////////////////////////////////////////////////////////////////
//	
//								Nano OS	Version	1.0
//						Copyright 2009 xxxxxxx, xxxxxxx
//	File:	NanoOS.h
//	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
//	Description: Header file for the NanoOS APIs
//	
///////////////////////////////////////////////////////////////////////////////

#ifndef _NANOOS_H
#define _NANOOS_H

#include "Nano_types.h"
#include "Nano_Task.h"

///////////////////////////////////////////////////////////////////////////////
// OS Error Codes
///////////////////////////////////////////////////////////////////////////////
typedef enum
{
	SUCCESS = 0,
	FAILED = 1,

	INSUFFICIENT_STACK = 10,
	INVALID_PERIOD = 11,
	INVALID_BUDGET = 12,
	INVALID_TASK = 13,
	EXCEEDS_MAX_CPU = 14,
	UCOS_ERROR = 15,
	ARGUMENT_ERROR = 16,
	OUT_OF_SPACE = 17,
	NO_DATA = 18,
	INVALID_PRIORITY = 19,
	OUT_OF_BOUNDS = 20,
	CHKPT_NOT_ENABLED = 21,
	CHKPT_FAIL = 22,
	INVALID_ARG = 23, 
	INVALID_DEADLINE = 24,

	UNKNOWN = 99	

} OS_Error;

void _disable_interrupt();
void _enable_interrupt();

#define  OS_ENTER_CRITICAL()	_disable_interrupt()
#define  OS_EXIT_CRITICAL()	_enable_interrupt()


///////////////////////////////////////////////////////////////////////////////
// Global Data
///////////////////////////////////////////////////////////////////////////////
extern volatile BOOL OS_IsRunning;

///////////////////////////////////////////////////////////////////////////////
// Task creation APIs
///////////////////////////////////////////////////////////////////////////////
OS_Error OS_CreatePeriodicTask(
	UINT32 period_in_us,
	UINT32 deadline_in_us,
	UINT32 budget_in_us,
	UINT32 phase_shift_in_us,
	UINT32 * stack,
	UINT32 stack_size_in_bytes,
	OS_PeriodicTask *task,
	void (*periodic_entry_function)(void *pdata),
	void *pdata);

OS_Error OS_CreateAperiodicTask(
	UINT16 priority,
	UINT32 * stack,
	UINT32 stack_size_in_bytes,
	OS_AperiodicTask *task,
	void (*task_entry_function)(void *pdata),
	void *pdata);


///////////////////////////////////////////////////////////////////////////////
// The following funcstion Initializes the OS data structures
///////////////////////////////////////////////////////////////////////////////
void OS_Init();

///////////////////////////////////////////////////////////////////////////////
// The following funcstion starts the OS scheduling
// Note that this function never returns
///////////////////////////////////////////////////////////////////////////////
void OS_Start();

///////////////////////////////////////////////////////////////////////////////
// The following function sleeps for the specified duration of time. 
///////////////////////////////////////////////////////////////////////////////
void OS_Sleep(UINT32 interval_in_us);

///////////////////////////////////////////////////////////////////////////////
// The below function, gets the total elapsed time since the beginning
// of the system in microseconds.
///////////////////////////////////////////////////////////////////////////////
UINT64 OS_GetElapsedTime();

///////////////////////////////////////////////////////////////////////////////
// The following function gets the total time taken by the current
// thread since the thread has begun. Note that this is not the global 
// time, this is just the time taken from only the current thread.
///////////////////////////////////////////////////////////////////////////////
UINT64 OS_GetThreadElapsedTime();



#endif // _NANOOS_H
