///////////////////////////////////////////////////////////////////////////////
//	
//								Nano OS	Version	1.0
//						Copyright 2009 xxxxxxx, xxxxxxx
//	File:	C6713_Nano_Task.h
//	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
//	Description: OS Task Related routines
//	
//	Note: This file implements a singly linked list. Singly linked lists are
// 	not efficient when we have to delete some items from the middle. We do
//  have this requirement in the case of user calling 'Sleep' method. However
//	just for the sake of 'Sleep' we are not going in for a doubly linked list
//	as 'Sleep' will be very rarely used. On the other hand maintaining a
//	doubly linked list will have more overhead for normal scheduling operations.
// 
///////////////////////////////////////////////////////////////////////////////

#ifndef _NANO_QUEUE_H
#define _NANO_QUEUE_H

///////////////////////////////////////////////////////////////////////////////
// Necessary Include Files
///////////////////////////////////////////////////////////////////////////////
#include "Nano_Types.h"	// Include common data types being used
#include "NanoOS.h"  // For Nano_Error definition

// Typedef for Queue node
typedef struct Nano_QueueNode Nano_QueueNode;
struct Nano_QueueNode
{	
	volatile Nano_QueueNode * next;   
	volatile UINT64 key;				
};

// Public typedefs
typedef struct
{
	volatile Nano_QueueNode * head;
	volatile Nano_QueueNode * tail;
	volatile UINT32 count;
} Nano_Queue;

///////////////////////////////////////////////////////////////////////////////
// Queue manipulaion function
///////////////////////////////////////////////////////////////////////////////

// Function to initialize the queue. 					 
OS_Error Nano_QueueInit(Nano_Queue * q);

// Function to insert an element into the queue. The key value determines the 
// location at which it will be inserted. This is a sorted queue on key value.
OS_Error Nano_QueueInsert(Nano_Queue * q, void * item, UINT64 key);

// Function to delete an item from the queue.
OS_Error Nano_QueueDelete(Nano_Queue * q, void * item);

// Function to get the first element from the Queue. 
OS_Error Nano_QueueGet(Nano_Queue * q, void ** item, UINT64 * key);

// Function to peek the first element from the Queue. 
OS_Error Nano_QueuePeek(Nano_Queue * q, void ** item, UINT64 * key);

#endif // _NANO_QUEUE_H
