///////////////////////////////////////////////////////////////////////////////
//	
//								Nano OS	Version	1.0
//						Copyright 2009 xxxxxxx, xxxxxxx
//	File:	Nano_Sem.h
//	Author: Sandeep Budanur (sbudanu@ncsu.edu)
//	Description: Implementation of the NanoOS Semaphore APIs
//	
///////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////
//Add function prototypes here
//mysem_init - initialises the semaphopre
//input : sem -  pointer of type OS_Sem
//        pShared - not used in current implementation
//        value - semaphore initial value
// returns: 0 for SUCCESS, non-zero for ERROR
///////////////////////////////////////////////////////////////////////
#include "NanoOS.h"
#include "Nano_Sem.h"

extern volatile void * g_current_task;
extern Nano_Queue g_ready_q;
extern Nano_Queue g_wait_q;
extern Nano_Queue g_ap_wait_q;


OS_Error OS_SemInit(OS_Sem *sem, INT16 pshared, UINT32 value)
{
	if(!sem)
		return ARGUMENT_ERROR;

	sem->count = value;
	//Nano_InitQ(&sem->queue);	
	Nano_QueueInit(&sem->queue);
	return SUCCESS;
}

OS_Error OS_SemWait(OS_Sem *sem)
{
	OS_PeriodicTask * cur_task = (OS_PeriodicTask *) g_current_task;
	
	if(!sem)
		return ARGUMENT_ERROR;

	
	while(1)
	{
		//disable interrupts
		OS_ENTER_CRITICAL();
			
		if(sem->count == 0)
		{
			//block the thread			
			Nano_QueueDelete(&g_ready_q, (void*)cur_task); //delete the current task from periodic tasks queue
			Nano_QueueInsert(&sem->queue, (void*)cur_task, cur_task->alarm_time); //add the current task to the semaphore's blocked queue
			OS_EXIT_CRITICAL();
			Nano_ReSchedule();			
				
		}	
		else
		{
			sem->count--;	
			OS_EXIT_CRITICAL();
			break;
		}
		
	}	

	return SUCCESS;
}

OS_Error OS_SemPost(OS_Sem *sem)
{
	OS_PeriodicTask* task = NULL;
	UINT64 key = 0;
	if(!sem)
		return ARGUMENT_ERROR;

	//disable interrupts
	OS_ENTER_CRITICAL();
	//sem->count++;
	if(sem->count == 0)
	{
		//unblock the task and remove from blocked queue		
		Nano_QueueGet(&sem->queue, (void**)&task, &key);//remove from the semaphore's blocked queue

		if(task)
		{
			Nano_QueueInsert(&g_ready_q,(void*)task,key);//place in the periodic task queue					
		}
	}
	sem->count++;
	OS_EXIT_CRITICAL();
	if(task)
	{
		Nano_ReSchedule();
	}

	return SUCCESS;
}

OS_Error OS_SemDestroy(OS_Sem *sem)
{
	if(!sem)
		return ARGUMENT_ERROR;

	sem->count = 0;
	Nano_QueueInit(&sem->queue);//set pointers to NULL
	return SUCCESS;
}

OS_Error OS_SemGetvalue(OS_Sem *sem, INT32* val)
{
	if(!sem)
		return ARGUMENT_ERROR;
	if(val)
		*val = sem->count;	

	return SUCCESS;
}

