///////////////////////////////////////////////////////////////////////////////
//	
//								Nano OS	Version	1.0
//						Copyright 2009 xxxxxxx, xxxxxxx
//	File:	C6713_Nano_Task.h
//	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
//	Description: OS Task Related routines
//	
///////////////////////////////////////////////////////////////////////////////

#ifndef _NANO_TASK_H
#define _NANO_TASK_H

#include "Nano_Types.h"

///////////////////////////////////////////////////////////////////////////////
// Constants
///////////////////////////////////////////////////////////////////////////////
#define ONE_KB	1024
#define ONE_MB	ONE_KB * ONE_KB

#define MIN_PERIOD		100	// 100 uSec
#define MIN_BUDGET		100 // 100uSec

///////////////////////////////////////////////////////////////////////////////
// Type of Task
///////////////////////////////////////////////////////////////////////////////
#define PERIODIC	1
#define APERIODIC	2

///////////////////////////////////////////////////////////////////////////////
// Task TCB 
///////////////////////////////////////////////////////////////////////////////

typedef struct OS_PeriodicTask OS_PeriodicTask;
struct OS_PeriodicTask
{
	// Attributes for maintaining a list/queue of threads.
	// IMPORTANT: Make sure that the below two attributes are the first two
	// attributes in the same order. This is done so that we can have a single
	// queue implementation for all types of objects as long as the below two
	// members are part of those objects.
	volatile OS_PeriodicTask * next;	// Do NOT REORDER THESE TWO MEMBERS
	volatile UINT64 alarm_time;	// To be used for maintaining a queue ordered based on this key value

	UINT32 *top_of_stack;	// Do NOT REORDER THIS MEMBER, THE OFFSET 'SP_OFFSET_IN_TCB' IS USED IN C6713_OS_Context.asm

	// Following members are attributes of tasks
	UINT8 type;	
	UINT16 id;

	UINT32 period;
	UINT32 deadline;
	UINT32 budget;
	UINT32 phase;
	
	UINT32 *stack;
	UINT32 stack_size;
	void (*task_function)(void *pdata);
	void *pdata;
	
	// Following members are used by the scheduling algorithm
	UINT32 remaining_budget;
	UINT64 accumulated_budget;
	UINT64 stored_release_time;
	UINT32 exec_count;
	UINT32 TBE_count;
	UINT32 dline_miss_count;
	//volatile UINT64 next_release_time;
};

typedef struct OS_AperiodicTask OS_AperiodicTask;
struct OS_AperiodicTask
{
	// Attributes for maintaining a list/queue of threads.
	// IMPORTANT: Make sure that the below two attributes are the first two
	// attributes in the same order. This is done so that we can have a single
	// queue implementation for all types of objects as long as the below two
	// members are part of those objects.
	volatile OS_AperiodicTask * next;	// Do NOT REORDER THESE TWO MEMBERS
	volatile UINT64 priority;	// To be used for maintaining a queue ordered based on this key value

	UINT32 *top_of_stack;	// Do NOT REORDER THIS MEMBER, THE OFFSET 'SP_OFFSET_IN_TCB' IS USED IN C6713_OS_Context.asm

	UINT8 type;	
	UINT16 id;
	UINT32 *stack;
	UINT32 stack_size;
	void (*task_function)(void *pdata);
	void *pdata;
};

#endif // _NANO_TASK_H
