;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	
;;								Nano OS	Version	1.0
;;						Copyright 2009 xxxxxxx, xxxxxxx
;;	File:	C6713_OS_Context.asm
;;	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
;;	Description: OS Task Stack & Context Switch routines
;;	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;********************************************************************************************************
;                                    PUBLIC and EXTERNAL REFERENCES
;********************************************************************************************************

FP	.set	A15
DP	.set	B14
SP	.set	B15	

SP_OFFSET_IN_TCB	.set	16

	.global	_C6713_BuildTaskStack
	.global _C6713_IntContextStore
	.global _C6713_ContextRestore
	.global	_g_current_task
	.global _INTERRUPT_STACK

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	THE SP MUST BE ALIGNED ON AN 8-BYTE BOUNDARY.  (Ref. Datasheet of C6711 DSP )                        
;;	WHICH means that the framesize has to be a multiple of 8-BYTE.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

_C6713_BuildTaskStack:		; UINT32 * C6713_BuildTaskStack(UINT32 * (A4), task_function (B4), task_arg (A6))
	ADDK	-8,A4   	; For C calling convention create 8 bytes space
	STW		B14, *--A4	; Push B14
	STW		B13, *--A4	; Push B13
	STW		B12, *--A4	; Push B12
	STW		B11, *--A4	; Push B11
	STW		B10, *--A4	; Push B10
	STW		B9, *--A4	; Push B9
	STW		B8, *--A4	; Push B8
	STW		B7, *--A4	; Push B7
	STW		B6, *--A4	; Push B6
	STW		B5, *--A4	; Push B5
	STW		B4, *--A4	; Push B4
	STW		B4, *--A4	; Create B3 value as the address of the task function to be returned
	STW		B2, *--A4	; Push B2
||  mvc		IER,B2						   ; B2 = IER
	STW		B1, *--A4	; Push B1
||  mvc		CSR,B1                         ; B1 = CSR
	STW		B0, *--A4	; Push B0
||  mvc		AMR,B0                         ; B0 = AMR

	STW		A15, *--A4	; Push A14
	STW		A14, *--A4	; Push A14
	STW		A13, *--A4	; Push A13
	STW		A12, *--A4	; Push A12
	STW		A11, *--A4	; Push A11
	STW		A10, *--A4	; Push A10
	STW		A9, *--A4	; Push A9
	STW		A8, *--A4	; Push A8
	STW		A7, *--A4	; Push A7
	STW		A6, *--A4	; Push A6
	STW		A5, *--A4	; Push A5
	STW		A6, *--A4	; Push A4	; Create A4 as if this is the task_arg when the task function is called
	STW		A3, *--A4	; Push A3
	STW		A2, *--A4	; Push A2
	STW		A1, *--A4	; Push A1
	STW		A0, *--A4	; Push A0
	OR		2, B2, B2	; Set NMIF bit
	STW		B2, *--A4	; Push IER
	OR		3, B1, B1	; Set GIE 
	STW		B1, *--A4	; Push CSR
	STW		B0, *--A4		; Push AMR, Dont have a post decrement, this value will be returned
	
	B 	B3
	NOP	5
	

_C6713_IntContextStore:
	STW		B14, *--SP	; Push B14
	STW		B13, *--SP	; Push B13
	STW		B12, *--SP	; Push B12
	STW		B11, *--SP	; Push B11
	STW		B10, *--SP	; Push B10
	STW		B9, *--SP	; Push B9
	STW		B8, *--SP	; Push B8
	STW		B7, *--SP	; Push B7
	STW		B6, *--SP	; Push B6
	STW		B5, *--SP	; Push B5
	STW		B4, *--SP	; Push B4
||	MVC		IRP, B4		; The return address when an interrupted has occurred
	STW		B4, *--SP	; Push B3 as the IRP value. When we are not in Interrupt, IRP is made to keep B3 value above (see _C6713_ContextStore)
	STW		B2, *--SP	; Push B2
||  MVC		IER,B2						   ; B2 = IER
	OR		2, B2, B2	; Set NMIF bit
	STW		B1, *--SP	; Push B1
||  MVC		CSR,B1                         ; B1 = CSR
	OR		3, B1, B1	; Set GIE 
	STW		B0, *--SP	; Push B0
||  MVC		AMR,B0                         ; B0 = AMR
	STW		A15, *--SP	; Push A14
	STW		A14, *--SP	; Push A14
	STW		A13, *--SP	; Push A13
	STW		A12, *--SP	; Push A12
	STW		A11, *--SP	; Push A11
	STW		A10, *--SP	; Push A10
	STW		A9, *--SP	; Push A9
	STW		A8, *--SP	; Push A8
	STW		A7, *--SP	; Push A7
	STW		A6, *--SP	; Push A6
	STW		A5, *--SP	; Push A5
	STW		A4, *--SP	; Push A4
	STW		A3, *--SP	; Push A3
	STW		A2, *--SP	; Push A2
	STW		A1, *--SP	; Push A1
	STW		A0, *--SP	; Push A0
	STW		B2, *--SP	; Push IER
	STW		B1, *--SP	; Push CSR
	STW		B0, *--SP	; Push AMR, Dont have a post decrement, this value will be returned
	
	; Update the latest stack pointer in current task's TCB
	MVKL	_g_current_task,B0	
	MVKH	_g_current_task,B0 
	LDW		*B0,B0
	NOP		4
	STW		SP, *+B0(SP_OFFSET_IN_TCB)
	
	; Return to the caller
	B 	B3
	NOP	5


_C6713_ContextRestore:		; void C6713_ContextRestore(new_thread_pointer)

	; We have to update the _g_current_task and stack pointer in task TCB
	; automically. i.e. there should not be any interrupt or anything in
	; between. Or else the whole think would fall apart !!
	; Anyway as part of ContextRestore CSR value will be restored to
	; what is required
	MVC CSR, B0
	CLR B0,0,0,B0	; Clear the GIE
	MVC B0,CSR

	; Update the _g_current_task to point to the new task
	MVKL	_g_current_task,B0	
	MVKH	_g_current_task,B0 
	STW		A4,*B0
	NOP    4

	; Update the latest stack pointer from the current task's TCB
	LDW		*+A4(SP_OFFSET_IN_TCB), SP
    NOP    4

	; Unwind the stack
	LDW		*SP++,B0
	NOP		4
	MVC		B0,AMR      ; AMR B0
	LDW		*SP++,B1
	NOP		4
	MVC		B1,CSR      ; CSR B1
	LDW		*SP++,B2
	NOP		4
	MVC		B2,IER      ; IER B2

	LDW		*SP++,A0
	LDW		*SP++,A1
	LDW		*SP++,A2
	LDW		*SP++,A3
	LDW		*SP++,A4
	LDW		*SP++,A5
	LDW		*SP++,A6
	LDW		*SP++,A7
	LDW		*SP++,A8
	LDW		*SP++,A9
	LDW		*SP++,A10
	LDW		*SP++,A11
	LDW		*SP++,A12
	LDW		*SP++,A13
	LDW		*SP++,A14
	LDW		*SP++,A15

	LDW		*SP++,B0
	LDW		*SP++,B1
	LDW		*SP++,B2
	LDW		*SP++,B3
	LDW		*SP++,B4
	LDW		*SP++,B5
	LDW		*SP++,B6
	LDW		*SP++,B7
	LDW		*SP++,B8
	LDW		*SP++,B9
	LDW		*SP++,B10
	LDW		*SP++,B11
	LDW		*SP++,B12
	LDW		*SP++,B13
	LDW		*SP++,B14
	
	B 	B3
	NOP	5

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Switches the context from one thread to another.
;; The g_current_task is updated as part of this function
;; to point to the new thread
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	.global _C6713_ContextSw
_C6713_ContextSw:	; void C6713_ContextSw(new_thread_pointer)

	STW		B14, *--SP	; Push B14
	STW		B13, *--SP	; Push B13
	STW		B12, *--SP	; Push B12
	STW		B11, *--SP	; Push B11
	STW		B10, *--SP	; Push B10
	STW		B9, *--SP	; Push B9
	STW		B8, *--SP	; Push B8
	STW		B7, *--SP	; Push B7
	STW		B6, *--SP	; Push B6
	STW		B5, *--SP	; Push B5
	STW		B4, *--SP	; Push B4
	STW		B3, *--SP	; Push B3
	STW		B2, *--SP	; Push B2
||  MVC		IER,B2						   ; B2 = IER
	STW		B1, *--SP	; Push B1
||  MVC		CSR,B1                         ; B1 = CSR
	STW		B0, *--SP	; Push B0
||  MVC		AMR,B0                         ; B0 = AMR

	STW		A15, *--SP	; Push A14
	STW		A14, *--SP	; Push A14
	STW		A13, *--SP	; Push A13
	STW		A12, *--SP	; Push A12
	STW		A11, *--SP	; Push A11
	STW		A10, *--SP	; Push A10
	STW		A9, *--SP	; Push A9
	STW		A8, *--SP	; Push A8
	STW		A7, *--SP	; Push A7
	STW		A6, *--SP	; Push A6
	STW		A5, *--SP	; Push A5
	STW		A4, *--SP	; Push A4
	STW		A3, *--SP	; Push A3
	STW		A2, *--SP	; Push A2
	STW		A1, *--SP	; Push A1
	STW		A0, *--SP	; Push A0

	STW		B2, *--SP	; Push IER
	STW		B1, *--SP	; Push CSR
	STW		B0, *--SP	; Push AMR, Dont have a post decrement, this value will be returned
	
	; We have to update the _g_current_task and stack pointer in task TCB
	; automically. i.e. there should not be any interrupt or anything in
	; between. Or else the whole think would fall apart !!
	; Anyway as part of ContextRestore CSR value will be restored to
	; what is required
	MVC CSR, B0
	CLR B0,0,0,B0	; Clear the GIE
	MVC B0,CSR

	; Update the latest stack pointer in current task's TCB
	MVKL	_g_current_task,B0	
	MVKH	_g_current_task,B0 
	LDW		*B0,B0
	NOP		4
	STW		SP, *+B0(SP_OFFSET_IN_TCB)

	; Update the g_current_task to point to the new task	
	MVKL	_g_current_task,B0	
	MVKH	_g_current_task,B0 
	STW		A4,*B0
	NOP    4

	; The new task is passed as an argument to this function.
	; Get the stack pointer from its TCB into SP
	LDW		*+A4(SP_OFFSET_IN_TCB), SP
	NOP    4

	; Unwind the stack
	LDW		*SP++,B0
	NOP		4
	MVC		B0,AMR      ; AMR B0
	LDW		*SP++,B1
	NOP		4
	MVC		B1,CSR      ; CSR B1
	LDW		*SP++,B2
	NOP		4
	MVC		B2,IER      ; IER B2

	LDW		*SP++,A0
	LDW		*SP++,A1
	LDW		*SP++,A2
	LDW		*SP++,A3
	LDW		*SP++,A4
	LDW		*SP++,A5
	LDW		*SP++,A6
	LDW		*SP++,A7
	LDW		*SP++,A8
	LDW		*SP++,A9
	LDW		*SP++,A10
	LDW		*SP++,A11
	LDW		*SP++,A12
	LDW		*SP++,A13
	LDW		*SP++,A14
	LDW		*SP++,A15

	LDW		*SP++,B0
	LDW		*SP++,B1
	LDW		*SP++,B2
	LDW		*SP++,B3
	LDW		*SP++,B4
	LDW		*SP++,B5
	LDW		*SP++,B6
	LDW		*SP++,B7
	LDW		*SP++,B8
	LDW		*SP++,B9
	LDW		*SP++,B10
	LDW		*SP++,B11
	LDW		*SP++,B12
	LDW		*SP++,B13
	LDW		*SP++,B14
	NOP	5
	B 	B3
	NOP	5

	.global _ChangeToIntContext
_ChangeToIntContext:
	
	; We will just change the SP to the bottom of interrupt stack. 
	; We assume no nesting of interrupts
	; Keep space for 8 bytes for C calling convention
	; Also remember that actual bottom of stack is _INTERRUPT_STACK - 4
	; Also keep the interrupt stack 8 byte aligned. So deduct 4 more
	MVKL	_INTERRUPT_STACK - 32,SP
	MVKH	_INTERRUPT_STACK - 32,SP 

	B 	B3
	NOP	5	
