;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;	
;;								Nano OS	Version	1.0
;;						Copyright 2009 xxxxxxx, xxxxxxx
;;	File:	C6713_OS_Init.asm
;;	Author:	Balasubramanya bhat (bbhat@ncsu.edu)
;;	Description: OS Initialization routines
;;	
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

OS_INTERRUPT_STACK_SIZE		.set	1024

SP	.set	B15	
DP	.set	B14

		.sect ".text"

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Symbol references
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;	
		.global ___bss__	; Beginning of BSS section
		.global ___end__	; Beginning of BSS section
		.global _c_int00	; beginning of C initialization
		.global INT_VECT_TABLE	; Beginning of Interrupt Service Vector Table
		.global __disable_interrupt
		.global __enable_interrupt
		.global RESET
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Function called by the reset vector
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	.global HANDLE_RESET
HANDLE_RESET:

		;; First setup the ISR Table address in ISTP (Interrupt service table pointer register)
        mvk INT_VECT_TABLE,B0	; Address of Interrupt Vector Table
        mvc B0,ISTP             ; set the value in ISTP

		;; First clear the bss section
CLEAR_BSS:
        mvkl ___bss__,B1        ; load destination function address to B1
        mvkh ___bss__,B1
        mvkl ___end__,B2        ; load destination function address to B2
        mvkh ___end__,B2  
		ZERO	B3		   
LOOP_CLEAR_BSS:
		CMPLT	B1, B2, B0
	[!B0] B		_c_int00
		NOP		5
		STW		B3, *B1++
		B		LOOP_CLEAR_BSS
		NOP		5

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; _disable_interrupt: function for disable interrupt                                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		.sect ".text"

__disable_interrupt: 
		mvc	CSR,B4
		and	1,B4,B0
 [!B0]	CLR B4,1,1,B4		
 [B0]	SET B4,1,1,B4
		CLR B4,0,0,B4
		mvc B4,CSR
		B 	B3
		NOP	5

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; _enable_interrupt: function for enable interrupt                                    
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
__enable_interrupt: 
		mvc	CSR,B4
		and	2,B4,B0
 [!B0]	CLR B4,0,0,B4		
 [B0]	SET B4,0,0,B4
 		mvc	B4,CSR
		B 	B3
		NOP	5

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;					 Initial the whole system
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		.global	_C6713_SystemInit
_C6713_SystemInit:  ; Initial the whole system

        MVK		0x0,B0

        mvc		B0,AMR
        mvc		B0,ISR
        mvc		B0,ICR
        mvc		B0,IER
        mvc		B0,IRP
        mvc		B0,ICR
        mvc		B0,NRP
        
        MVK		0x0100,B0	
        mvc		B0,CSR

        mvkl	RESET,B0
        mvkh	RESET,B0
        mvc		B0,ISTP


        MVK		0x0A0,A0
        MVK 	0x0A1,A1
        MVK 	0x0A2,A2
        MVK		0x0A3,A3
        MVK  	0x0A4,A4
        MVK		0x0A5,A5
        MVK		0x0A6,A6
        MVK		0x0A7,A7
        MVK		0x0A8,A8
        MVK		0x0A9,A9
        MVK		0x0A10,A10
        MVK		0x0A11,A11
        MVK		0x0A12,A12
        MVK		0x0A13,A13
        MVK		0x0A14,A14
;       MVK		0x0,A15  ; Frame pointer

        MVK     .S2     0x0B0,B0
        MVK     .S2     0x0B1,B1
        MVK     .S2     0x0B2,B2
;       MVK		0x0B3,B3  Return address
        MVK     .S2     0x0B4,B4
        MVK     .S2     0x0B5,B5
        MVK     .S2     0x0B6,B6
        MVK     .S2     0x0B7,B7
        MVK     .S2     0x0B8,B8
        MVK     .S2     0x0B9,B9
        MVK     .S2     0x0B10,B10
        MVK     .S2     0x0B11,B11
        MVK     .S2     0x0B12,B12
        MVK     .S2     0x0B13,B13

        B		B3
        NOP		5

		
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Interrupt Stack
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
_INTERRUPT_STACK_TOP	.usect ".int_stack", OS_INTERRUPT_STACK_SIZE, 4
